﻿
''' <summary>
''' Takes an equation and converts it to Reverse Polish Notation and evaulates the equation.
''' </summary>
''' <remarks></remarks>
Public Class RPNCalculator

    Private _equation As String = String.Empty
    Private _rpnEquation As String = String.Empty
    Private _replaceVariable As Char = Nothing


    Public ReadOnly Property Equation As String

        Get
            Return _equation
        End Get

    End Property

    Public ReadOnly Property RPNEquation As String

        Get
            Return _rpnEquation
        End Get

    End Property


    ''' <summary>
    ''' Initializes the RPN Class, passing in an equation with a variable.
    ''' </summary>
    ''' <param name="_equ">The equation to process.</param>
    ''' <param name="_var">The variable character.</param>
    ''' <remarks></remarks>
    Public Sub New(ByVal _equ As String, ByVal _var As Char)

        'String builder
        Dim _strBld As New System.Text.StringBuilder()

        'AppendString
        Dim _dataList As New List(Of String)

        'backup
        Dim _back As String = String.Empty

        'Check each character
        For Each _ch As Char In _equ

            'Add it together
            If Char.IsNumber(_ch) Then _back &= _ch.ToString Else _dataList.Add(_back) : _dataList.Add(_ch.ToString) : _back = String.Empty

        Next

        'One last check
        If Not _back.Equals(String.Empty) Then _dataList.Add(_back) : _back = String.Empty

        'Now spit it back out properly
        For Each _str As String In _dataList
            If Not _str.Equals(String.Empty) Then _strBld.AppendFormat("{0} ", _str)
        Next
        

        'Assign back
        _equ = _strBld.ToString.TrimEnd

        'Do a bit of messing
        Dim _equList As New List(Of String)

        For Each ch As Char In _equ.ToCharArray
            _equList.Add(ch.ToString)
        Next


        'Loop
        For i As Integer = 0 To _equList.Count - 1
            If _equList(i).Equals(_var) Then

                'Parenthesis Check
                If _equList.Count >= i + 3 Then If _equList(i + 2).Equals("("c) Then _equList(i) = String.Format("{0} *", _equList(i))
                If Not (i - 2) <= -1 Then If _equList(i - 2).Equals(")"c) Then _equList(i) = String.Format("* {0}", _equList(i))

                'Number check
                If _equList.Count >= i + 3 Then If New TokenHelper().IsNumber(_equList(i + 2)) Then _equList(i) = String.Format("{0} *", _equList(i))
                If Not (i - 2) <= -1 Then If New TokenHelper().IsNumber(_equList(i - 2)) Then _equList(i) = String.Format("* {0}", _equList(i))

            End If
        Next

        _strBld = New System.Text.StringBuilder
        For Each _str As String In _equList
            _strBld.Append(_str)
        Next

        'Store our original equation
        _equation = _strBld.ToString

        'Now convert it to RPN
        _rpnEquation = New InfixTransformer().Transform(_equation)

        'Store the character replacer
        _replaceVariable = _var

    End Sub

    ''' <summary>
    ''' Initializes the RPN Class, passing in an equation with a variable.
    ''' </summary>
    ''' <param name="_equ">The equation to process.</param>
    ''' <remarks></remarks>
    Public Sub New(ByVal _equ As String)

        'Store our original equation
        _equation = _equ

        'Now convert it to RPN
        _rpnEquation = New InfixTransformer().Transform(_equ)

    End Sub

    ''' <summary>
    ''' Evaluates the function at the given value.
    ''' </summary>
    ''' <param name="_int">The number to evaluate the function at.</param>
    ''' <returns>The value of this function</returns>
    ''' <remarks></remarks>
    ''' 
    Public Function Evaluate(ByVal _int As Double) As Double

        'Create our helper
        Dim _th As New TokenHelper

        'Number list
        Dim _numberList As New List(Of Double)

        'Split
        Dim _list As New List(Of String)(_rpnEquation.Split(" "c))

        'Start looping
        For Each ch As String In _list

            'check for a blank
            If Not ch.Equals(" "c) Then

                'check this
                If ch.Equals(_replaceVariable) Then ch = _int

                'Check
                If _th.IsNumber(ch) Then

                    'It's a number, add it.
                    _numberList.Add(Convert.ToDouble(ch))

                ElseIf _th.IsOperator(ch) Then

                    If ch.Equals("!"c) Then

                        'Perform
                        _numberList(_numberList.Count - 1) = Factoral(_numberList(_numberList.Count - 1))

                    Else

                        'It's an operator, dequeue the previous items
                        Dim _first As Double = Convert.ToDouble(_numberList(_numberList.Count - 2))
                        Dim _second As Double = Convert.ToDouble(_numberList(_numberList.Count - 1))

                        'Create our stored number
                        Dim _storedNumber As Double = 0

                        'Perform our operation
                        Select Case ch
                            Case "+"
                                _storedNumber = _first + _second
                            Case "-"
                                _storedNumber = _first - _second
                            Case "/"
                                _storedNumber = _first / _second
                            Case "^"
                                _storedNumber = _first ^ _second
                            Case "*"
                                _storedNumber = _first * _second
                            Case "!"
                                '_storedNumber = Factoral(_first)
                        End Select

                        'Remove the two we just used
                        _numberList.RemoveRange(_numberList.Count - 2, 2)

                        'Put back into the stack
                        _numberList.Add(_storedNumber)

                    End If

                End If

            End If

        Next

        'Give it back.
        Return _numberList(0)

    End Function

    ''' <summary>
    ''' Performs a factoral operation.
    ''' </summary>
    ''' <param name="_number">The number to perform the factoral on.</param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Private Function Factoral(ByVal _number As Integer) As Double

        'Store
        Dim _return As Double = 1

        'Loop
        For i As Integer = _number To 1 Step -1
            _return *= i
        Next

        'Give
        Return _return

    End Function

    ''' <summary>
    ''' Evaluates the function.
    ''' </summary>
    ''' <returns>The value of this function</returns>
    ''' <remarks></remarks>
    Public Function Evaluate() As Double
        Return Evaluate(0)
    End Function

End Class


Public Class InfixTransformer

    Private _tokenHelper As New TokenHelper
    Private _output As New Queue(Of String)
    Private _operators As New Stack(Of String)

    ''' <summary>
    ''' Converts an Infix Equation to a PostFix Equation.
    ''' </summary>
    ''' <param name="expression">The expression to use, minus any spaces.</param>
    ''' <returns>A PostFix Equation.</returns>
    ''' <remarks></remarks>
    Public Function Transform(ByVal expression As String) As String

        'Tokens
        Dim tokens As New List(Of String)(expression.Split(" "c))

        'Remove the bad ones
        For i As Integer = tokens.Count - 1 To 0 Step -1
            If tokens(i).Equals(String.Empty) Then tokens.RemoveAt(i)
        Next

        'Loop
        For Each token As String In tokens

            If Not token.Equals(" "c) Then

                If _tokenHelper.IsNumber(token) Then
                    'Number
                    _output.Enqueue(token)

                ElseIf _tokenHelper.IsFunction(token) Then
                    'Function
                    _operators.Push(token)

                ElseIf token.Equals(","c) Then
                    While _operators.Count > 0 AndAlso Not _operators.Peek.Equals("("c)


                        Dim topOperator As String = _operators.Pop
                        _output.Enqueue(topOperator)

                    End While
                ElseIf _tokenHelper.IsOperator(token) Then

                    While _operators.Count > 0 AndAlso _tokenHelper.IsOperator(_operators.Peek)

                        If (_tokenHelper.IsLeftAssociative(token) AndAlso _operators.Count > 0 AndAlso _tokenHelper.GetPrecedenceFor(token) <= _tokenHelper.GetPrecedenceFor(_operators.Peek)) OrElse (_tokenHelper.IsRightAssociative(token) AndAlso _tokenHelper.GetPrecedenceFor(token) < _tokenHelper.GetPrecedenceFor(_operators.Peek)) Then

                            Dim operatorToReturn As String = _operators.Pop()
                            _output.Enqueue(operatorToReturn)

                        Else

                            Exit While

                        End If

                    End While

                    _operators.Push(token)

                End If

                If token.Equals("("c) Then

                    _operators.Push(token)

                End If

                If token.Equals(")"c) Then

                    While _operators.Count > 0 AndAlso Not _operators.Peek.Equals("("c)
                        _output.Enqueue(_operators.Pop)
                    End While

                    _operators.Pop()

                End If

                If _operators.Count > 0 AndAlso _tokenHelper.IsFunction(_operators.Peek) Then

                    _output.Enqueue(_operators.Pop)

                End If

            Else

                _output.Enqueue(" ")

            End If

        Next

        While _operators.Count > 0 AndAlso _tokenHelper.IsOperator(_operators.Peek)
            _output.Enqueue(_operators.Pop)
        End While

        Dim _result As String = String.Empty
        While _output.Count > 0
            _result &= _output.Dequeue & " "
        End While
        Return _result.TrimEnd

    End Function

End Class

Public Class TokenHelper

    Private _operators As String = "+-*/%=!^"
    Private _leftAssociativeOperators As String = "*/%+-"
    Private _rightAssociativeOperators As String = "!=^"

    Private ReadOnly Property _operatorsToPrecedenceMapping As Dictionary(Of String, Integer)

        Get

            'Build the dictionary
            Dim _dic As New Dictionary(Of String, Integer)
            _dic.Add("=", 1)
            _dic.Add("+-", 2)
            _dic.Add("*/%", 3)
            _dic.Add("!", 4)
            _dic.Add("^", 5)

            'Give back
            Return _dic

        End Get

    End Property

    ''' <summary>
    ''' Checks to see if a token is an operator.
    ''' </summary>
    ''' <param name="token">The token.</param>
    ''' <returns>Boolean</returns>
    ''' <remarks></remarks>
    Public Function IsOperator(ByVal token As String) As Boolean
        If String.IsNullOrEmpty(token) Then Return False
        Return _operators.Contains(token)
    End Function

    ''' <summary>
    ''' Returns the precedence for the passed token.
    ''' </summary>
    ''' <param name="token">The token to pass.</param>
    ''' <returns>Integer</returns>
    ''' <remarks></remarks>
    Public Function GetPrecedenceFor(ByVal token As String) As Integer
        For Each _key As String In _operatorsToPrecedenceMapping.Keys
            If _key.Contains(token) Then Return _operatorsToPrecedenceMapping(_key)
        Next
        Throw New InvalidOperationException("invalid operator")
    End Function

    ''' <summary>
    ''' Returns if the passed token is a number.
    ''' </summary>
    ''' <param name="token">The token.</param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Function IsNumber(ByVal token As String) As Boolean
        Dim _x As Double
        If Double.TryParse(token, _x) Then Return True Else Return False
        'Return Char.IsNumber(Convert.ToChar(token))
    End Function

    ''' <summary>
    ''' Returns if the token
    ''' </summary>
    ''' <param name="token"></param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Function IsFunction(ByVal token As String) As Boolean
        Return Char.IsLetter(Convert.ToChar(token))
    End Function

    ''' <summary>
    ''' Returns if the token is left associative
    ''' </summary>
    ''' <param name="token">the token</param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Function IsLeftAssociative(ByVal token As String) As Boolean
        If Not String.IsNullOrEmpty(token) AndAlso token.Length.Equals(1) Then Return _leftAssociativeOperators.Contains(token) Else Throw New ArgumentException("incorrect token!")
    End Function

    ''' <summary>
    ''' Returns if the token is right associative.
    ''' </summary>
    ''' <param name="token">the token</param>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Function IsRightAssociative(ByVal token As String) As Boolean
        If Not String.IsNullOrEmpty(token) AndAlso token.Length.Equals(1) Then Return _rightAssociativeOperators.Contains(token) Else Throw New ArgumentException("incorrect token!")
    End Function

End Class
