﻿Module Integration

    ''' <summary>
    ''' Main Sub that controls everything.
    ''' </summary>
    ''' <remarks></remarks>
    Sub Main()

        While True

            'Title
            Console.Title = "Integration Approximation Program"

            'Clear
            Console.Clear()

            'Decide a method
            Console.WriteLine("Which approximation method do you want to use?")
            Console.WriteLine("----------------------------------------------")
            Console.WriteLine("A.) Left Riemann Sum [Rectangles]")
            Console.WriteLine("B.) Midpoint Riemann Sum [Rectangles]")
            Console.WriteLine("C.) Right Riemann Sum [Rectangles]")
            Console.WriteLine("D.) Simpson's Method [Parabola, very accurate]")
            Console.WriteLine("E.) Trapezoidal Method [Trapezoids, pretty accurate]")
            Console.WriteLine("F.) All together")
            Console.WriteLine("G.) Exit")
            Console.WriteLine("----------------------------------------------")

            'Get our answer
            Dim _response As ConsoleKeyInfo = Console.ReadKey

            'Switch
            Select Case _response.Key

                Case ConsoleKey.A

                    Console.Title = "Left Riemann Sum"

                    'Clear
                    Console.Clear()

                    'Performing a left Riemann Sum
                    Dim _answer As Decimal = LeftRiemannApproximation(New RPNCalculator(GetEquation, GetVariable), GetRange, GetIntervals)

                    'Write
                    Console.WriteLine("The approximation of the function is: {0}", _answer)
                    Console.WriteLine("Press any key to continue...")
                    Console.ReadKey()

                Case ConsoleKey.B

                    Console.Title = "Midpoint Riemann Sum"

                    'Clear
                    Console.Clear()

                    'Performing a midpoint riemann sum
                    Dim _answer As Decimal = MidPointRiemannApproximation(New RPNCalculator(GetEquation, GetVariable), GetRange, GetIntervals)

                    Console.WriteLine("The approximation of the function is: {0}", _answer)
                    Console.WriteLine("Press any key to continue...")
                    Console.ReadKey()

                Case ConsoleKey.C

                    Console.Title = "Right Riemann Sum"

                    'Clear
                    Console.Clear()

                    'Performing a Right Riemann Sum
                    Dim _answer As Decimal = RightRiemannApproximation(New RPNCalculator(GetEquation, GetVariable), GetRange, GetIntervals)

                    'Write
                    Console.WriteLine("The approximation of the function is: {0}", _answer)
                    Console.WriteLine("Press any key to continue...")
                    Console.ReadKey()

                Case ConsoleKey.D

                    Console.Title = "Simpson's Rule Approximation"

                    'Clear
                    Console.Clear()

                    'Performing a Simpson's Approximation
                    Dim _answer As Decimal = SimpsonApproximation(New RPNCalculator(GetEquation, GetVariable), GetRange, GetIntervals)

                    'Write
                    Console.WriteLine("The approximation of the function is: {0}", _answer)
                    Console.WriteLine("Press any key to continue...")
                    Console.ReadKey()

                Case ConsoleKey.E

                    Console.Title = "Trapezoidal Approximation"

                    'Clear
                    Console.Clear()

                    'Performing a Trapezoidal Approximation
                    Dim _answer As Decimal = TrapezoidalApproximation(New RPNCalculator(GetEquation, GetVariable), GetRange, GetIntervals)

                    'Write
                    Console.WriteLine("The approximation of the function is: {0}", _answer)
                    Console.WriteLine("Press any key to continue...")
                    Console.ReadKey()

                Case ConsoleKey.F

                    Console.Title = "Group Approximation"

                    'Clear
                    Console.Clear()

                    'Get the variables
                    Dim _calc As New RPNCalculator(GetEquation, GetVariable)

                    'Get the range
                    Dim _range() As String = GetRange()

                    'Get the intervals
                    Dim _interval As String = GetIntervals()

                    'Clear again
                    Console.Clear()

                    'Get answers
                    Dim _answer1 As Decimal = LeftRiemannApproximation(_calc, _range, _interval)
                    Dim _answer2 As Decimal = MidPointRiemannApproximation(_calc, _range, _interval)
                    Dim _answer3 As Decimal = RightRiemannApproximation(_calc, _range, _interval)
                    Dim _answer4 As Decimal = TrapezoidalApproximation(_calc, _range, _interval)
                    Dim _answer5 As Decimal = SimpsonApproximation(_calc, _range, _interval)

                    'Clear a last time
                    Console.Clear()

                    'Write them all out
                    Console.WriteLine("Approximation for '{0}'", _calc.Equation)
                    Console.WriteLine("Range: {0} <= X <= {1}", _range(0), _range(1))
                    Console.WriteLine("Intervals: {0}", _interval)
                    Console.WriteLine("-----------------------------------------")

                    Console.WriteLine("Left Riemann Approximation: {0}", _answer1)
                    Console.WriteLine("MidPoint Riemann Approximation: {0}", _answer2)
                    Console.WriteLine("Right Riemann Approximation: {0}", _answer3)
                    Console.WriteLine("Trapezoidal Approximation: {0}", _answer4)
                    Console.WriteLine("Simpson Rule Approximation: {0}", _answer5)
                    Console.WriteLine()
                    Console.WriteLine("Average Approximation: {0}", (_answer1 + _answer2 + _answer3 + _answer4 + _answer5) / 5)
                    Console.WriteLine("-----------------------------------------")
                    Console.WriteLine("Press any key to continue...")
                    Console.ReadKey()

                Case ConsoleKey.G

                    'Leave
                    Exit While

            End Select

        End While

    End Sub

    ''' <summary>
    ''' Gets our equation from the user then returns control back to the Main() Sub.
    ''' </summary>
    ''' <returns>String</returns>
    ''' <remarks></remarks>
    Private Function GetEquation() As String

        'Ask to get the equation
        Console.WriteLine("Type in the equation you wish to approximate. Example: x^2")

        'Create the equation
        Dim _equation As String = Console.ReadLine

        'Blank space
        Console.WriteLine()

        'Now give it back
        Return _equation

    End Function

    ''' <summary>
    ''' Gets our variable from the user then returns control back to the Main() Sub.
    ''' </summary>
    ''' <returns>String</returns>
    ''' <remarks></remarks>
    Private Function GetVariable() As String

        'Display
        Console.WriteLine("What variable is used in the equation? Example: x^2 has the 'x' variable.")

        'Get
        Dim _var As String = Console.ReadLine

        'Blank line
        Console.WriteLine()

        'We only want one character.
        Return _var(0)

    End Function

    ''' <summary>
    ''' Gets the Range that the function should go from.
    ''' </summary>
    ''' <returns>String Array</returns>
    ''' <remarks></remarks>
    Private Function GetRange() As String()

        'Define our array
        Dim _arr(1) As String

        'Now ask
        Console.WriteLine("What is the lower bound?")
        _arr(0) = Console.ReadLine()

        Console.WriteLine("What is the upper bound?")
        _arr(1) = Console.ReadLine()

        'Blank line
        Console.WriteLine()

        'TryParse
        Dim _tp As Decimal

        'Make sure they're numbers
        If Decimal.TryParse(_arr(0), _tp) AndAlso Decimal.TryParse(_arr(1), _tp) Then Return _arr Else Throw New ArgumentException("One of the passed ranges is NOT a number! TRY AGAIN!")

    End Function

    ''' <summary>
    ''' Gets the number of subintervals to be used.
    ''' </summary>
    ''' <returns>String</returns>
    ''' <remarks></remarks>
    Private Function GetIntervals() As String

        'Get the interval
        Console.WriteLine("How many intervals do you wish to have? [If you are using Simpsons and this is not even, the program will throw an error!].")

        'Get
        Dim _intervals As String = Console.ReadLine

        'Now check
        Return _intervals
        'If (CInt(_intervals) Mod 2).Equals(0) Then Return _intervals Else Throw New ArgumentException("The given interval is NOT even!")

    End Function


    ''' <summary>
    ''' Performs a Simpson Approximation of the given function.
    ''' </summary>
    ''' <param name="_calc">A reference to the RPN calculator.</param>
    ''' <param name="_range">The range to perform the approximation on.</param>
    ''' <param name="_interval">The step or change for each function.</param>
    ''' <returns>An approximation of the area of the function.</returns>
    ''' <remarks>The most accurate method here.</remarks>
    Private Function SimpsonApproximation(ByVal _calc As RPNCalculator, ByVal _range As String(), ByVal _interval As String) As Decimal

        'Stopwatch
        Dim _s As New Stopwatch

        'Clear
        Console.Clear()

        'Convert our items
        Dim _lowerBound As Decimal = CDec(_range(0))
        Dim _upperBound As Decimal = CDec(_range(1))
        Dim _nInterval As Decimal = CDec(_interval)

        'Check
        If Not (_nInterval Mod 2).Equals(0) Then Throw New ArgumentException("You must have an even number of sub-intervals for the Simpson Approximation!")

        'Begin timing
        _s.Start()

        'Now calculate our change
        Dim _steps As Decimal = (_upperBound - _lowerBound) / _nInterval


        'Go
        Console.WriteLine("Performing a Simpson Approximation on the function '{0}'", _calc.Equation)
        Console.WriteLine("{0} <= X <= {1}", _lowerBound, _upperBound)
        Console.WriteLine("Number of Intervals: {0}", _nInterval)
        Console.WriteLine()

        'Calculate the 'beginning'
        Dim _head As Decimal = (_upperBound - _lowerBound) / (3 * _nInterval)

        'calculate the 'loop header'
        Dim _loopHead As Double = _calc.Evaluate(_lowerBound)

        'calculate the 'loop footer'
        Dim _loopFooter As Double = _calc.Evaluate(_upperBound)

        'Create a switch
        Dim _switch As Boolean = False

        'Create our loop storing data
        Dim _loopData As Double = 0

        'Begin our loop to calculate the crap inbetween the header and footer
        For i As Decimal = _lowerBound + _steps To _upperBound - _steps Step +_steps

            'Create our data
            _loopData += If(_switch, 2, 4) * _calc.Evaluate(i)
            _switch = Not _switch

        Next

        'Add our header, footer, and data
        Dim _totalLoop As Decimal = _loopHead + _loopData + _loopFooter

        'Multiply it by the head.
        _totalLoop *= _head

        'Stop
        _s.Stop()

        'Report
        Console.WriteLine("Calculatons took approximately {0} milliseconds.", _s.Elapsed.TotalMilliseconds)
        Console.WriteLine()

        'Give it all
        Return _totalLoop

    End Function

    ''' <summary>
    ''' Performs a Left Riemann Sum of the given function.
    ''' </summary>
    ''' <param name="_calc">A reference to the RPN calculator.</param>
    ''' <param name="_range">The range to perform the approximation on.</param>
    ''' <param name="_interval">The step or change for each function.</param>
    ''' <returns>A number that approximates the area under the curve.</returns>
    ''' <remarks></remarks>
    Private Function LeftRiemannApproximation(ByVal _calc As RPNCalculator, ByVal _range As String(), ByVal _interval As String) As Decimal

        'Stopwatch
        Dim _s As New Stopwatch

        'Start
        _s.Start()

        'Clear
        Console.Clear()

        'Convert our items
        Dim _lowerBound As Decimal = CDec(_range(0))
        Dim _upperBound As Decimal = CDec(_range(1))
        Dim _nInterval As Decimal = CDec(_interval)

        'Now calculate our change
        Dim _steps As Decimal = (_upperBound - _lowerBound) / _nInterval

        'Go
        Console.WriteLine("Performing a Left Riemann Approximation on the function '{0}'", _calc.Equation)
        Console.WriteLine("{0} <= X <= {1}", _lowerBound, _upperBound)
        Console.WriteLine("Number of Intervals: {0}", _nInterval)
        Console.WriteLine()

        'Setup a variable to store our inside approximations
        Dim _approximation As Decimal = 0

        'Setup our loop
        For i As Double = _lowerBound To _upperBound - _steps Step +_steps

            'Now begin adding data
            _approximation += _calc.Evaluate(i)

        Next

        'One last step
        _approximation *= _steps

        'Stop
        _s.Stop()

        'Report
        Console.WriteLine("Calculatons took approximately {0} milliseconds.", _s.Elapsed.TotalMilliseconds)
        Console.WriteLine()

        'Return Q multipled by the total sum of the approximations
        Return _approximation

    End Function

    ''' <summary>
    ''' Performs a MidPoint Riemann Sum of the given function.
    ''' </summary>
    ''' <param name="_calc">A reference to the RPN calculator.</param>
    ''' <param name="_range">The range to perform the approximation on.</param>
    ''' <param name="_interval">The step or change for each function.</param>
    ''' <returns>A number that approximates the area under the curve.</returns>
    ''' <remarks></remarks>
    Private Function MidPointRiemannApproximation(ByVal _calc As RPNCalculator, ByVal _range As String(), ByVal _interval As String) As Decimal

        'Stopwatch
        Dim _s As New Stopwatch

        'Start
        _s.Start()

        'Clear
        Console.Clear()

        'Convert our items
        Dim _lowerBound As Decimal = CDec(_range(0))
        Dim _upperBound As Decimal = CDec(_range(1))
        Dim _nInterval As Decimal = CDec(_interval)

        'Now calculate our change
        Dim _steps As Decimal = (_upperBound - _lowerBound) / _nInterval

        'Go
        Console.WriteLine("Performing a Midpoint Riemann Approximation on the function '{0}'", _calc.Equation)
        Console.WriteLine("{0} <= X <= {1}", _lowerBound, _upperBound)
        Console.WriteLine("Number of Intervals: {0}", _nInterval)
        Console.WriteLine()

        'total
        Dim _total As Double = 0

        'Start looping
        For i As Decimal = _lowerBound To _upperBound Step _steps

            'Find the midpoint
            Dim _midPoint As Double = ((i + _steps) + i) / 2

            'Now find the height
            Dim _height As Double = _calc.Evaluate(_midPoint)

            'Add it to the total
            _total += _height

        Next

        'Final step
        _total *= _steps

        'Stop
        _s.Stop()

        'Report
        Console.WriteLine("Calculatons took approximately {0} milliseconds.", _s.Elapsed.TotalMilliseconds)
        Console.WriteLine()

        'Give back
        Return _total

    End Function

    ''' <summary>
    ''' Performs a Left Riemann Sum of the given function.
    ''' </summary>
    ''' <param name="_calc">A reference to the RPN calculator.</param>
    ''' <param name="_range">The range to perform the approximation on.</param>
    ''' <param name="_interval">The step or change for each function.</param>
    ''' <returns>A number that approximates the area under the curve.</returns>
    ''' <remarks></remarks>
    Private Function RightRiemannApproximation(ByVal _calc As RPNCalculator, ByVal _range As String(), ByVal _interval As String) As Decimal

        'Stopwatch
        Dim _s As New Stopwatch

        'Start
        _s.Start()

        'Clear
        Console.Clear()

        'Convert our items
        Dim _lowerBound As Decimal = CDec(_range(0))
        Dim _upperBound As Decimal = CDec(_range(1))
        Dim _nInterval As Decimal = CDec(_interval)

        'Now calculate our change
        Dim _steps As Decimal = (_upperBound - _lowerBound) / _nInterval

        'Go
        Console.WriteLine("Performing a Right Riemann Approximation on the function '{0}'", _calc.Equation)
        Console.WriteLine("{0} <= X <= {1}", _lowerBound, _upperBound)
        Console.WriteLine("Number of Intervals: {0}", _nInterval)
        Console.WriteLine()

        'Setup a variable to store our inside approximations
        Dim _approximation As Decimal = 0

        'Setup our loop
        For i As Double = _lowerBound + _steps To _upperBound Step +_steps

            'Now begin adding data
            _approximation += _calc.Evaluate(i)

        Next

        'Final step
        _approximation *= _steps

        'Stop
        _s.Stop()

        'Report
        Console.WriteLine("Calculatons took approximately {0} milliseconds.", _s.Elapsed.TotalMilliseconds)
        Console.WriteLine()

        'Return total
        Return _approximation

    End Function

    ''' <summary>
    ''' Performs a Trapezoidal approximation of the given function.
    ''' </summary>
    ''' <param name="_calc">A reference to the RPN calculator.</param>
    ''' <param name="_range">The range to perform the approximation on.</param>
    ''' <param name="_interval">The step or change for each function.</param>
    ''' <returns>A number that approximates the area under the curve.</returns>
    ''' <remarks></remarks>
    Private Function TrapezoidalApproximation(ByVal _calc As RPNCalculator, ByVal _range As String(), ByVal _interval As String) As Decimal

        'Stopwatch
        Dim _s As New Stopwatch

        'Start
        _s.Start()

        'Clear
        Console.Clear()

        'Convert our items
        Dim _lowerBound As Decimal = CDec(_range(0))
        Dim _upperBound As Decimal = CDec(_range(1))
        Dim _nInterval As Decimal = CDec(_interval)

        'Now calculate our change
        Dim _steps As Decimal = (_upperBound - _lowerBound) / _nInterval

        'Go
        Console.WriteLine("Performing a Trapezoidal Approximation on the function '{0}'", _calc.Equation)
        Console.WriteLine("{0} <= X <= {1}", _lowerBound, _upperBound)
        Console.WriteLine("Number of Intervals: {0}", _nInterval)
        Console.WriteLine()

        'Total to hold our value
        Dim _total As Decimal = 0

        'Start looping
        For i As Decimal = _lowerBound To _upperBound - _steps Step _steps

            'First side
            Dim _s1 As Decimal = _calc.Evaluate(i)

            'Second
            Dim _s2 As Decimal = _calc.Evaluate(i + _steps)


            'Now perform
            _total += ((_s1 + _s2) / 2)

        Next

        'final step
        _total *= (_steps)

        Console.WriteLine("Calculatons took approximately {0} milliseconds.", _s.Elapsed.TotalMilliseconds)
        Console.WriteLine()

        'Return total
        Return _total

    End Function

End Module
